; Version: 2013-01-26
; Written for LM1802

; 
; This program is using the principle that A, B and C acts like arguments to 
; subroutines, and the data is not guaranteed to be the same after the
; subrotine end. 
; Register X, Y and Z must however always be returned as they were sent in. 
; Other registers are undefined. 


; 
; Inits the LEM1802 display by finding the 
; monitor hardware index. The VRAM is 
; placed where the A register is pointing at. 
; If A equals to 0 the monitor will disconnect. 
; 
; The monitor index will be conveniently be saved to 
; [LEM_INDEX] if succeded, otherwise 0xffff.
; Also saves the handy a handy pointer [LEM_VRAM] to 
; where it's allocated. 
; 

:LEM_INDEX DAT 0x0000
:LEM_VRAM DAT 0x0000 
:LEM_FONT DAT 0x0000

:init_lem1802 
    SET [LEM_VRAM], A  ; Save the pointer to VRAM start, given as a parameter. 
    HWN I          ; Store the number of connected hardware devices 
    
    :init_lem_loop ; Weird name to avoid name collisions 
        SUB I, 1 
        SET [LEM_INDEX], I  ; Every loop save I to [LEM_INDEX]

        ; Overflow happened, meaning all the hw-indices has been searched. 
        IFE I, 0xFFFF   
            SET PC, init_end  ; 0xffff has already been saved to [MEMORY_INDEX], end loop. 
    
        HWQ I
        IFE A, 0xF615    ; The monitor has been found
            SET PC, init_end  ; End the loop

    SET PC, init_lem_loop

    :init_end
        SET A, 0
        SET B, [LEM_VRAM]
        HWI [LEM_INDEX]      ; Initialize the monitor VRAM using the given pointer. 
        SET PC, POP          ; Return to where it was called from. 


        
:LEM_COLOR DAT 0xf000 

; Prints a character stored in register A, using using the foreground color 
; and background color stored in the 8 first MSB in color, in VRAM-index B.
:print_char 
     AND A, 0x007F  ; Zero all bits except the 7 LSB 
     BOR A, [LEM_COLOR]
     ADD B, [LEM_VRAM]
     SET [B], A
     SET PC, POP 


; Prints the char A using B and C as X and Y-values instead. 
:print_char2 
	MUL C, 32 
	ADD B, C
	JSR print_char 
	SET PC, POP

; Like print_char but prints it with swapped foreground and background colors. 
:print_char_s
    SET PUSH, A    ; Push A and B to stack
    SET PUSH, B
    SET A, [LEM_COLOR]  
    AND A, 0xf000         ; Store the 4 first MSBs in A 
    SHR A, 4              ; Shift them to the right 
    SET B, [LEM_COLOR]    
    AND B, 0x0f00         ; Store the second group of 4 bits in B 
    SHL B, 4              ; Shift them left 
    BOR A, B              ; Merge them together in A 
    
    SET B, POP            ; POP the coord
    ADD B, [LEM_VRAM]
    BOR A, POP            ; POP the char and store merge with color. 
    SET [B], A
    SET PC, POP
    
; A subroutine that prints the current. Usefull when using a custom one. 
:print_font
    SET PUSH, X 
    SET X, 0 
    :print_font_loop 
        SET A, X
        SET B, X
        JSR print_char
        ADD X, 1
        IFG X, 128
            SET PC, print_font_end 
    SET PC, print_font_loop
    :print_font_end
    SET X, POP
    SET PC, POP


; A subroutine that prints the character in A over the entire screen. 
; Usefull for clearing the screen by using a full sized character. 
:fill_screen
    SET PUSH, X
    SET PUSH, Y
    SET X, A
    SET Y, 0
    :fill_screen_loop
        SET A, X
        SET B, Y
        JSR print_char 
        ADD Y, 1
        IFG Y, 0x180 
            SET PC, fill_screen_end 
        SET PC, fill_screen_loop
    :fill_screen_end
    
    SET Y, POP
    SET X, POP
    SET PC, POP



